/*
**	File:		01 DBCC loginfo.sql
**
**	Summary:	Run in a job to run indirect checkpoints prior to SQL 2012
**				set the percentage limit before running.
**
**	SQL Server Versions:	2005 onwards
**	----------------------------------------------------------------------------
**	Written by Rainer Unwin (c) 2014
**
**	For more scripts check out http://www.rmuniwn.com
**
**	You may alter this code for your own non-commercial purposes (e.g. in a
**	for-sale commercial tool).
**	You may republish altered code as long as you include this copyright and
**	give due credit, however you must obtain my prior permission before blogging
**	this code.
**
**	this code and information are provided "as is" without warranty of
**	any kind, either expressed or implied, including but not limited
**	to the implied warranties of merchantability and/or fitness for a
**	particular purpose. You use the script at your own risk and should
**	always test in a pre-production or test environment before running
**	in production.
*/
use [master];
go

-- Remove existing database
if exists (select name from sys.databases where name = N'tlogbasics') drop database tlogbasics;
go

-- Create a new database with 2 * 4MB log growth for 8 VLFs (Virtual Log Files)
create database tlogbasics
 ON  PRIMARY 
( NAME = N'tlogbasics', FILENAME = N'C:\SQLData\DataFiles\tlogbasics.mdf' , SIZE = 5120KB , FILEGROWTH = 1024KB )
 LOG ON 
( NAME = N'tlogbasics_log', FILENAME = N'C:\SQLData\LogFiles\tlogbasics_log.ldf' , SIZE = 4096KB , FILEGROWTH = 1024KB);
go
alter database tlogbasics modify file ( NAME = N'tlogbasics_log', SIZE = 8192KB );
go

use tlogbasics;
go

-- Shows structure of VLFs. Points to note follow command
dbcc loginfo(tlogbasics);
go
/*
RecoveryUnitId - new in 2012, not covered
FileID - The file given in below sys.database_files command
FileSize - The size, in bytes, of the VLF
StartOffset - The number of bytes into the file that the VLF starts
	- Also note that there are 8KB left free at the start of the file, this is a partial copy of the
		database boot page, run the below command to see the boot page
		DBCC PAGE('tlogbasics', 1, 9, 3) WITH TABLERESULTS, NO_INFOMSGS;
FSeqNo - the current number of the VLF, an incrementing number to track where in the log you are
Status - Active (Value: 2) or not active and (Re)usable (Value: 0)
Parity - Tracks where in a VLF we are up to (Shows current parity) when the log is read (During
		Recovery, mirroring, etc) you are at the end of the log when a log block has a different
		parity to this field, permissible values below:
		0 - Never been used
		64 - Value for when the VLF has been used an odd number of times
		128 - Value for when the VLF has been used an even number of times
CreateLSN - The Log Sequence Number (LSN) when the VLFs were created.
*/

select *
from sys.database_files;