/*
**	File:		05 page splits and log usage.sql
**
**	Summary:	Step through the script to see how much log a page split
**				can generate. This could be worse in case of a root split
**
**	SQL Server Versions:	2005 onwards
**	----------------------------------------------------------------------------
**	Written by Rainer Unwin (c) 2014
**
**	For more scripts check out http://www.rmuniwn.com
**
**	You may alter this code for your own non-commercial purposes (e.g. in a
**	for-sale commercial tool).
**	You may republish altered code as long as you include this copyright and
**	give due credit, however you must obtain my prior permission before blogging
**	this code.
**
**	this code and information are provided "as is" without warranty of
**	any kind, either expressed or implied, including but not limited
**	to the implied warranties of merchantability and/or fitness for a
**	particular purpose. You use the script at your own risk and should
**	always test in a pre-production or test environment before running
**	in production.
*/
use tempdb;
go

-- create a table to use, with a cluster
if object_id('dbo.pagesplit', 'U') is not null drop table dbo.pagesplit;
go

create table dbo.pagesplit (
	id int
	, data char(10) default 'a'
);
go

create unique clustered index cix_pagesplit on dbo.pagesplit (id);
go

-- insert a row to avoid the overhead of table creation
insert into dbo.pagesplit (id) values (1);
go

-- now get the size of a single row insert, note the size
begin transaction

	insert into dbo.pagesplit (id) values (2);

	select database_transaction_log_bytes_used
	from sys.dm_tran_database_transactions t
		inner join sys.dm_exec_requests r
			on r.transaction_id = t.transaction_id
	where r.session_id = @@spid;

commit transaction;
go
-- log bytes generated by simple insert: 

-- now fill the page
;with nums (n)
as (
	select 1 union all select 1 union all select 1 union all select 1 union all select 1 union all 
	select 1 union all select 1 union all select 1 union all select 1 union all select 1
), fill (n)
as (
	select top (350) row_number() over(order by n.n) + 3
	from nums n, nums n1, nums n2
)
insert into dbo.pagesplit (id)
select n
from fill;
go

-- now insert the missing record (3) and lets see the result of a page split
begin transaction

	insert into dbo.pagesplit (id) values (3);

	select database_transaction_log_bytes_used
	from sys.dm_tran_database_transactions t
		inner join sys.dm_exec_requests r
			on r.transaction_id = t.transaction_id
	where r.session_id = @@spid;

commit transaction;
go
-- log bytes generated by simple insert: 
-- WOW! How many times more log?
select [put the last value here] / [put the top value here].0

--
-- What if we had a wider row? what do you think? why?
--
if object_id('dbo.pagesplit', 'U') is not null drop table dbo.pagesplit;
go

create table dbo.pagesplit (
	id int
	, data char(100) default 'a'
);
go

create unique clustered index cix_pagesplit on dbo.pagesplit (id);
go

insert into dbo.pagesplit (id) values (1);
go

-- now get the size of a single row insert, note the size
begin transaction

	insert into dbo.pagesplit (id) values (2);

	select database_transaction_log_bytes_used
	from sys.dm_tran_database_transactions t
		inner join sys.dm_exec_requests r
			on r.transaction_id = t.transaction_id
	where r.session_id = @@spid;

commit transaction;
go
-- log bytes generated by simple insert: 

;with nums (n)
as (
	select 1 union all select 1 union all select 1 union all select 1 union all select 1 union all 
	select 1 union all select 1 union all select 1 union all select 1 union all select 1
), fill (n)
as (
	select top (69) row_number() over(order by n.n) + 3
	from nums n, nums n1, nums n2
)
insert into dbo.pagesplit (id)
select n
from fill;

-- now insert the missing record (3) and lets see the result of a page split
begin transaction

	insert into dbo.pagesplit (id) values (3);

	select database_transaction_log_bytes_used
	from sys.dm_tran_database_transactions t
		inner join sys.dm_exec_requests r
			on r.transaction_id = t.transaction_id
	where r.session_id = @@spid;

commit transaction;
go
-- log bytes generated by simple insert: 
-- How many times more log this time?
select [put the last value here] / [put the top value here].0

-- So wider rows cause less log during page splits!